#include <stdlib.h>
#include "integerio.h"

#define STACKSIZE 1024


/* If stack is full, print an error message,
   otherwise push the value onto the stack */
#define PUSH(value) \
if(sp < STACKSIZE) \
{ \
   stack[sp] = (value);\
   sp++;\
}\
else\
{\
   printf("Stack overflow error\n");\
}


/* If stack is empty, print an error message and "break",
   otherwise pop the top value into varname */
#define POP_OR_FAIL(varname) \
if(sp > 0)\
{\
   sp--;\
   (varname) = stack[sp];\
}\
else\
{\
   printf("Stack underflow error\n");\
   break;\
}

int main(void)
{
   int num, arg1, arg2, i;
   int stack[STACKSIZE];
   int sp = 0, in_base = 10, out_base = 10;
   
   SkipSpace();
   while(LookChar()!=EOF)
   {
      if(int_available(in_base))
      {
	 num = read_int_base(in_base);
	 PUSH(num);
      }
      else
      {  /* Operator! */
	 switch(GetChar())
	 {
	 case 'o':
	       POP_OR_FAIL(num);
	       if(num < 2 || num >16)
	       {
		  printf("Only bases 2-16 (decimal) supported\n");
	       }
	       else
	       {
		  out_base = num;
	       }
	       break;
	 case 'i':
	       POP_OR_FAIL(num);
	       if(num < 2 || num >16)
	       {
		  printf("Only bases 2-16 (decimal) supported\n");
	       }
	       else
	       {
		  in_base = num;
	       }
	       break;
	 case 'p':
	       POP_OR_FAIL(num);
	       write_int_base(num,out_base);
	       putchar('\n');
	       PUSH(num);
	       break;	       
	 case '+':
	       POP_OR_FAIL(arg2);
	       POP_OR_FAIL(arg1);
	       num = arg1+arg2;
	       PUSH(num);
	       break;
	 case '-':
	       POP_OR_FAIL(arg2);
	       POP_OR_FAIL(arg1);
	       num = arg1-arg2;
	       PUSH(num);
	       break;
	 case '*':
	       POP_OR_FAIL(arg2);
	       POP_OR_FAIL(arg1);
	       num = arg1*arg2;
	       PUSH(num);
	       break;
	 case '/':
	       POP_OR_FAIL(arg2);
	       POP_OR_FAIL(arg1);
	       num = arg1/arg2;
	       PUSH(num);
	       break;
	 case '%':
	       POP_OR_FAIL(arg2);
	       POP_OR_FAIL(arg1);
	       num = arg1%arg2;
	       PUSH(num);
	       break;
	 case 'S':
	       for(i=0; i<sp; i++)
	       {
		  write_int_base(stack[i],out_base);
		  putchar(' ');
	       }
	       putchar('\n');
	       break;
	 case 'P':
	       printf("Input base (decimal): %d  Output Base (decimal): %d\n",
		     in_base,out_base);
	       break;
	 default:
	       printf("Unknown operand\n");
	 }
      }
      SkipSpace();
   }   
   return EXIT_SUCCESS;
}





